package de.jottyfan.quickiemod;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.jottyfan.quickiemod.block.ModBlocks;
import de.jottyfan.quickiemod.blockentity.ModBlockentity;
import de.jottyfan.quickiemod.event.EventBlockBreak;
import de.jottyfan.quickiemod.feature.ModFeatures;
import de.jottyfan.quickiemod.item.ModItems;
import de.jottyfan.quickiemod.itemgroup.ModItemGroup;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.event.player.PlayerBlockBreakEvents;
import net.fabricmc.fabric.api.loot.v3.LootTableEvents;
import net.fabricmc.fabric.api.registry.CompostingChanceRegistry;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.Items;
import net.minecraft.loot.LootPool;
import net.minecraft.loot.condition.SurvivesExplosionLootCondition;
import net.minecraft.loot.entry.ItemEntry;

/**
 *
 * @author jotty
 *
 */
public class Quickiemod implements ModInitializer {
	public static final String MOD_ID = "quickiemod";
	public static final Logger LOGGER = LoggerFactory.getLogger(MOD_ID);

	private void registerComposterItems() {
		CompostingChanceRegistry.INSTANCE.add(ModItems.ITEM_COTTONSEED, 0.5f);
		CompostingChanceRegistry.INSTANCE.add(ModItems.ITEM_COTTON, 0.75f);
		CompostingChanceRegistry.INSTANCE.add(ModItems.ITEM_CANOLASEED, 0.5f);
		CompostingChanceRegistry.INSTANCE.add(ModItems.ITEM_CANOLA, 0.75f);
	}

	private void registerLootTableChanges() {
		LootTableEvents.MODIFY.register((key, tableBuilder, source, registries) -> {
			if (source.isBuiltin()) {
				if ("minecraft:blocks/short_grass".equals(key.getValue().toString())) {
					tableBuilder.pool(LootPool.builder()
							.with(ItemEntry.builder(ModItems.ITEM_COTTONSEED).weight(1))
							.with(ItemEntry.builder(Items.AIR).weight(4))
							.conditionally(SurvivesExplosionLootCondition.builder()));
				} else if ("minecraft:blocks/tall_grass".equals(key.getValue().toString())) {
					tableBuilder.pool(LootPool.builder()
							.with(ItemEntry.builder(ModItems.ITEM_CANOLASEED).weight(1))
							.with(ItemEntry.builder(Items.AIR).weight(7))
							.conditionally(SurvivesExplosionLootCondition.builder()));
				}
			}
		});
	}

	@Override
	public void onInitialize() {
		ModBlockentity.registerModBlockentities();
		List<Item> items = ModItems.registerModItems();
		List<Block> blocks = ModBlocks.registerModBlocks();
		ModFeatures.registerFeatures();
		registerComposterItems();
		registerLootTableChanges();
		ModItemGroup.registerItemGroup(items, blocks);
		PlayerBlockBreakEvents.AFTER.register((world, player, pos, state, blockEntity) -> {
			Block oldBlock = state.getBlock();
			new EventBlockBreak().doBreakBlock(world, pos, state, player, oldBlock);
		});
	}
}