package de.jottyfan.quickiemod.block;

import com.mojang.serialization.MapCodec;

import de.jottyfan.quickiemod.blockentity.BlockStackerEntity;
import de.jottyfan.quickiemod.blockentity.ModBlockentity;
import net.minecraft.block.AbstractBlock;
import net.minecraft.block.BlockEntityProvider;
import net.minecraft.block.BlockRenderType;
import net.minecraft.block.BlockState;
import net.minecraft.block.BlockWithEntity;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.block.entity.BlockEntityTicker;
import net.minecraft.block.entity.BlockEntityType;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.registry.RegistryKey;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.screen.ScreenHandler;
import net.minecraft.util.ActionResult;
import net.minecraft.util.Identifier;
import net.minecraft.util.ItemScatterer;
import net.minecraft.util.hit.BlockHitResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.world.World;

/**
 *
 * @author jotty
 *
 */
public class BlockStacker extends BlockWithEntity implements BlockEntityProvider {

	private final Direction source;
	private final Direction dest;

	public BlockStacker(Identifier identifier, Direction source, Direction dest) {
		super(AbstractBlock.Settings.create().hardness(2.5f).registryKey(RegistryKey.of(RegistryKeys.BLOCK, identifier)));
		this.source = source;
		this.dest = dest;
	}

	@Override
	public BlockRenderType getRenderType(BlockState state) {
		return BlockRenderType.MODEL;
	}

	/**
	 * define the source offset
	 *
	 * @return the direction of the source offset (1 block beside)
	 */
	public Direction getSourceOffset() {
		return source;
	}

	/**
	 * define the dest offset
	 *
	 * @return the direction of the dest offset (1 block beside)
	 */
	public Direction getDestOffset() {
		return dest;
	}

	@Override
	public BlockEntity createBlockEntity(BlockPos pos, BlockState state) {
		return new BlockStackerEntity(pos, state);
	}

	@Override
	protected MapCodec<? extends BlockWithEntity> getCodec() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public <T extends BlockEntity> BlockEntityTicker<T> getTicker(World world, BlockState state,
			BlockEntityType<T> type) {
		return validateTicker(type, ModBlockentity.BLOCKSTACKER_BLOCKENTITY,
				(world1, pos, state1, be) -> BlockStackerEntity.tick(world1, pos, state1, be));
	}

	@Override
	protected ActionResult onUse(BlockState state, World world, BlockPos pos, PlayerEntity player, BlockHitResult hit) {
//		if (!world.isClient) {
//			NamedScreenHandlerFactory screenHandlerFactory = state.createScreenHandlerFactory(world, pos);
//			if (screenHandlerFactory != null) {
//				player.openHandledScreen(screenHandlerFactory);
//			}
//		}
		return ActionResult.SUCCESS;
	}

	@Override
	public void onStateReplaced(BlockState state, World world, BlockPos pos, BlockState newState, boolean moved) {
		if (state.getBlock() != newState.getBlock()) {
			BlockEntity blockEntity = world.getBlockEntity(pos);
			if (blockEntity instanceof BlockStackerEntity) {
				ItemScatterer.spawn(world, pos, (BlockStackerEntity) blockEntity);
				// update comparators
				world.updateComparators(pos, this);
			}
			super.onStateReplaced(state, world, pos, newState, moved);
		}
	}

	@Override
	public boolean hasComparatorOutput(BlockState state) {
		return true;
	}

	@Override
	public int getComparatorOutput(BlockState state, World world, BlockPos pos) {
		return ScreenHandler.calculateComparatorOutput(world.getBlockEntity(pos));
	}
}
