package de.jottyfan.quickiemod.blockentity;

import java.util.ArrayList;
import java.util.List;

import de.jottyfan.quickiemod.block.BlockDrill;
import net.minecraft.block.BlockState;
import net.minecraft.block.Blocks;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.world.World;

/**
 *
 * @author jotty
 *
 */
public class DrillBlockEntity extends BlockEntity {

	private static final Integer MAXDRILLSTEP = 20;

	private Integer drillstep;
	private final Integer maxDrillStep;

	public DrillBlockEntity(BlockPos pos, BlockState state) {
		super(ModBlockentity.DRILL_BLOCKENTITY, pos, state);
		this.maxDrillStep = MAXDRILLSTEP;
	}

	public static void tick(World world, BlockPos pos, BlockState state, BlockEntity be) {
		if (be instanceof DrillBlockEntity dbe) {
			Direction dir = state.get(BlockDrill.DIRECTION);
			List<BlockPos> list = new ArrayList<>();
			if (Direction.DOWN.equals(dir)) {
				list = downFrom(pos);
		  } else if (Direction.EAST.equals(dir)) {
				list = directedFrom(pos.east());
			}	else if (Direction.SOUTH.equals(dir)) {
				list = directedFrom(pos.south());
			}	else if (Direction.WEST.equals(dir)) {
				list = directedFrom(pos.west());
			}	else if (Direction.NORTH.equals(dir)) {
				list = directedFrom(pos.north());
			}
			DrillBlockEntity.tick(world, pos, state, dbe, MAXDRILLSTEP, list);
		}
	}

	public static final List<BlockPos> directedFrom(BlockPos pos) {
//		Quickiemod.LOGGER.info("directed");
		List<BlockPos> list = new ArrayList<>();
		list.add(pos);
		list.add(pos.up());
		list.add(pos.up().up());
		list.add(pos.up().up().up());
		list.add(pos.down()); // must be last position
		return list;
	}

	public static final List<BlockPos> downFrom(BlockPos pos) {
//		Quickiemod.LOGGER.info("down");
		List<BlockPos> list = new ArrayList<>();
		Integer tracesMod = pos.getY() % 8;
		tracesMod = tracesMod < 0 ? tracesMod * -1 : tracesMod; // lower that 0 makes it negative
		if (tracesMod != 0) {
			list.add(pos.north());
		}
		if (tracesMod != 1) {
			list.add(pos.north().west());
		}
		if (tracesMod != 2) {
			list.add(pos.west());
		}
		if (tracesMod != 3) {
			list.add(pos.south().west());
		}
		if (tracesMod != 4) {
			list.add(pos.south());
		}
		if (tracesMod != 5) {
			list.add(pos.south().east());
		}
		if (tracesMod != 6) {
			list.add(pos.east());
		}
		if (tracesMod != 7) {
			list.add(pos.north().east());
		}
		list.add(pos.down()); // must be last position
		return list;
	}

	protected static final void moveBlockWithEntity(BlockPos from, BlockPos to, World world) {
		BlockEntity be = world.getBlockEntity(from);
		BlockState bs = world.getBlockState(from);
		if (be != null) {
			world.setBlockState(from, Blocks.AIR.getDefaultState());
			Integer newFuel = bs.get(BlockDrill.FUEL) - 1;
			world.setBlockState(to, bs.with(BlockDrill.FUEL, newFuel));
			world.removeBlockEntity(from);
		}
	}

	protected static final Boolean drill(BlockPos pos, List<BlockPos> toList, World world) {
		Boolean lastSuccess = false;
		for (BlockPos to : toList) {
			if (!world.getBlockState(to).isOf(Blocks.BEDROCK)) {
				world.breakBlock(to, true);
				lastSuccess = pos.down() != to; // no need for the falling one
			} else {
				lastSuccess = false; // in case that the last one is a bedrock
			}
		}
		return lastSuccess;
	}

	public static void tick(World world, BlockPos pos, BlockState state, DrillBlockEntity be, Integer maxDrillStep,
			List<BlockPos> drillPosition) {
		if (state.get(BlockDrill.FUEL) > 0) {
			if (be.getDrillstep() < 1) {
				be.setDrillstep(maxDrillStep);
				if (drill(pos, drillPosition, world)) {
					moveBlockWithEntity(pos, drillPosition.get(drillPosition.size() - 1), world);
				}
			} else {
				be.doDrillstep();
			}
		}
	}

	public void doDrillstep() {
		setDrillstep(getDrillstep() - 1);
	}

	public Integer getDrillstep() {
		return drillstep == null ? maxDrillStep : drillstep;
	}

	public void setDrillstep(Integer drillstep) {
		this.drillstep = drillstep;
	}
}
